@@ -1,7 +1,6 @@
 import logging
 import signal
 import sys
-import time
 from argparse import ArgumentParser
 
 try:
@@ -64,10 +63,38 @@ def argument_parser():
     parser.add_argument(
         "--sweep-sec",
         dest="sweep_sec",
-        type=intx,
+        type=float,
         default=30,
         help="Set sweep_sec [default=%(default)r]",
     )
+    parser.add_argument(
+        "--tune-step-fft",
+        dest="tune_step_fft",
+        type=int,
+        default=0,
+        help="tune FFT step (0 is use sweep_sec) [default=%(default)r]",
+    )
+    parser.add_argument(
+        "--skip-tune-step",
+        dest="skip_tune_step",
+        type=int,
+        default=0,
+        help="skip FFT samples on retune [default=%(default)r]",
+    )
+    parser.add_argument(
+        "--sample_dir",
+        dest="sample_dir",
+        type=str,
+        default="",
+        help="where to write samples/FFT points",
+    )
+    parser.add_argument(
+        "--write_samples",
+        dest="write_samples",
+        default=0,
+        type=int,
+        help="if > 0, write FFT/raw samples to --sample_dir",
+    )
     parser.add_argument(
         "--nfft",
         dest="nfft",
@@ -118,11 +145,32 @@ def argument_parser():
         help="seconds to wait for healthy freq updates",
     )
     parser.add_argument(
-        "--retune-intervals",
-        dest="retune_intervals",
+        "--tuneoverlap",
+        dest="tuneoverlap",
+        type=float,
+        default=0.5,
+        help="multiple of samp_rate when retuning",
+    )
+    parser.add_argument(
+        "--inference_plan_file",
+        dest="inference_plan_file",
+        type=str,
+        default="",
+        help="full path to plan file for wavelearner",
+    )
+    parser.add_argument(
+        "--inference_output_dir",
+        dest="inference_output_dir",
+        type=str,
+        default="",
+        help="directory for inference output",
+    )
+    parser.add_argument(
+        "--inference_input_len",
+        dest="inference_input_len",
         type=int,
-        default=1,
-        help="if > 1, then schedule future retuning events in batches",
+        default=2048,
+        help="vector length for wavelearner",
     )
     return parser
 
@@ -147,9 +195,21 @@ def main():
 
     # ensure tuning tags arrive on FFT window boundaries.
     if options.samp_rate % options.nfft:
-        print("NFFT must be a factor of sample rate")
+        print("NFFT should be a factor of sample rate")
+
+    if options.write_samples and not options.sample_dir:
+        print("Must provide --sample_dir when writing samples/points")
         sys.exit(1)
 
+    wavelearner = None
+    try:
+        import wavelearner as wavelearner_lib  # pytype: disable=import-error
+
+        wavelearner = wavelearner_lib
+        print("using wavelearner")
+    except ModuleNotFoundError:
+        print("wavelearner not available")
+
     prom_vars = init_prom_vars()
     prom_vars["freq_start_hz"].set(options.freq_start)
     prom_vars["freq_end_hz"].set(options.freq_end)
@@ -167,7 +227,16 @@ def main():
         sdr=options.sdr,
         sdrargs=options.sdrargs,
         fft_size=options.nfft,
+        tune_overlap=options.tuneoverlap,
+        tune_step_fft=options.tune_step_fft,
+        skip_tune_step=options.skip_tune_step,
+        sample_dir=options.sample_dir,
+        write_samples=options.write_samples,
+        inference_plan_file=options.inference_plan_file,
+        inference_output_dir=options.inference_output_dir,
+        inference_input_len=options.inference_input_len,
         iqtlabs=iqtlabs,
+        wavelearner=wavelearner,
     )
 
     def sig_handler(_sig=None, _frame=None):
