@@ -3,8 +3,9 @@ import concurrent.futures
 import json
 import logging
 import os
+import pathlib
 import subprocess
-import sys
+import tempfile
 import threading
 import time
 
@@ -29,11 +30,12 @@ from gamutrf.sigwindows import get_center
 from gamutrf.sigwindows import graph_fft_peaks
 from gamutrf.sigwindows import parse_freq_excluded
 from gamutrf.sigwindows import scipy_find_sig_windows
+from gamutrf.sigwindows import ROLLING_FACTOR
 from gamutrf.utils import rotate_file_n, SCAN_FRES
 
 MB = int(1.024e6)
 FFT_BUFFER_TIME = 1
-BUFF_FILE = "/dev/shm/scanfftbuffer.txt.zst"  # nosec
+BUFF_FILE = "scanfftbuffer.txt.zst"  # nosec
 PEAK_TRIGGER = int(os.environ.get("PEAK_TRIGGER", "0"))
 PIN_TRIGGER = int(os.environ.get("PIN_TRIGGER", "17"))
 if PEAK_TRIGGER == 1:
@@ -168,9 +170,8 @@ def update_prom_vars(peak_dbs, new_bins, old_bins, prom_vars):
         old_bins_prom.labels(bin_freq=obin).inc()
 
 
-def process_fft(args, prom_vars, ts, fftbuffer, lastbins, running_df):
+def process_fft(args, scan_config, prom_vars, df, lastbins, running_df, last_dfs):
     global PEAK_DBS
-    df = pd.DataFrame(fftbuffer, columns=["ts", "freq", "db"])
     # resample to SCAN_FRES
     # ...first frequency
     df["freq"] = (df["freq"] / SCAN_FRES).round() * SCAN_FRES / 1e6
@@ -179,7 +180,7 @@ def process_fft(args, prom_vars, ts, fftbuffer, lastbins, running_df):
     df["db"] = df.groupby(["freq"])["db"].mean()
     df = df.reset_index().drop_duplicates(subset=["freq"])
     df = df.sort_values("freq")
-    df = calc_db(df)
+    df = calc_db(df, args.db_rolling_factor)
     freqdiffs = df.freq - df.freq.shift()
     mindiff = freqdiffs.min()
     maxdiff = freqdiffs.max()
@@ -202,7 +203,7 @@ def process_fft(args, prom_vars, ts, fftbuffer, lastbins, running_df):
     peak_dbs = {}
     bin_freq_count = prom_vars["bin_freq_count"]
     last_bin_freq_time = prom_vars["last_bin_freq_time"]
-    freq_start_mhz = args.freq_start / 1e6
+    freq_start_mhz = scan_config["freq_start"] / 1e6
     signals = scipy_find_sig_windows(
         df, width=args.width, prominence=args.prominence, threshold=args.threshold
     )
@@ -232,8 +233,17 @@ def process_fft(args, prom_vars, ts, fftbuffer, lastbins, running_df):
 
     if args.fftgraph:
         rotate_file_n(args.fftgraph, args.nfftgraph)
-        graph_fft_peaks(args.fftgraph, df, mean_running_df, sample_count_df, signals)
+        graph_fft_peaks(
+            args.fftgraph,
+            df,
+            mean_running_df,
+            sample_count_df,
+            signals,
+            last_dfs,
+            scan_config,
+        )
 
+    ts = df["ts"].max()
     for peak_freq, peak_db in signals:
         center_freq = get_center(
             peak_freq, freq_start_mhz, args.bin_mhz, args.record_bw_msps
@@ -262,7 +272,7 @@ def process_fft(args, prom_vars, ts, fftbuffer, lastbins, running_df):
     if old_bins:
         logging.info("old bins: %s", sorted(old_bins))
     update_prom_vars(peak_dbs, new_bins, old_bins, prom_vars)
-    return monitor_bins
+    return (monitor_bins, df)
 
 
 def recorder_req(recorder, recorder_args, timeout):
@@ -317,19 +327,26 @@ def zstd_file(uncompressed_file):
 
 
 def process_fft_lines(
-    args, prom_vars, buff_file, executor, proxy_result, runonce=False
+    args,
+    prom_vars,
+    buff_file,
+    executor,
+    proxy_result,
+    live_file,
 ):
     lastfreq = 0
-    fftbuffer = []
+    fftbuffer = None
     lastbins_history = []
     lastbins = set()
     frame_counter = prom_vars["frame_counter"]
     txt_buf = ""
     last_fft_report = 0
     fft_packets = 0
-    max_scan_pos = 0
+    last_sweep_start = 0
     context = zstandard.ZstdDecompressor()
     running_df = None
+    last_dfs = []
+    scan_config = None
 
     while True:
         if os.path.exists(args.log):
@@ -341,11 +358,13 @@ def process_fft_lines(
         openlogts = int(time.time())
         with open(args.log, mode=mode, encoding="utf-8") as l:
             while True:
+                if not live_file.exists():
+                    return
                 if not proxy_result.running():
                     logging.error(
                         "FFT proxy stopped running: %s", proxy_result.result()
                     )
-                    sys.exit(-1)
+                    return
                 now = int(time.time())
                 if now - last_fft_report > FFT_BUFFER_TIME * 2:
                     logging.info(
@@ -380,50 +399,78 @@ def process_fft_lines(
                     txt_buf = last_line
                     lines = lines[:-1]
                 try:
-                    df = pd.DataFrame(
-                        [line.strip().split() for line in lines],
-                        columns=["ts", "freq", "pw"],
-                        dtype=float,
-                    )
+                    records = []
+                    for line in lines:
+                        line = line.strip()
+                        record = json.loads(line)
+                        ts = float(record["ts"])
+                        sweep_start = float(record["sweep_start"])
+                        buckets = record["buckets"]
+                        scan_config = record["config"]
+                        records.extend(
+                            [
+                                {
+                                    "ts": ts,
+                                    "freq": float(freq),
+                                    "db": float(db),
+                                    "sweep_start": sweep_start,
+                                }
+                                for freq, db in buckets.items()
+                            ]
+                        )
+                    df = pd.DataFrame(records)
                 except ValueError as err:
                     logging.error(str(err))
                     continue
-                df = df[(df.pw > 0) & (df.pw <= args.max_raw_power)]
-                df = df[(df.freq >= args.freq_start) & (df.freq <= args.freq_end)]
                 df = df[(now - df.ts).abs() < 60]
-                df["scan_pos"] = (df.freq - args.freq_start) / (
-                    args.freq_end - args.freq_start
-                )
+                freq_start = scan_config["freq_start"]
+                freq_end = scan_config["freq_end"]
                 if df.size:
                     lastfreq = df.freq.iat[-1]
+                max_sweep_start = df["sweep_start"].max()
                 rotatelognow = False
-                for row in df.itertuples():
-                    rollover = row.scan_pos < 0.1 and max_scan_pos > 0.9 and fftbuffer
-                    max_scan_pos = max(max_scan_pos, row.scan_pos)
-                    if rollover:
-                        max_scan_pos = row.scan_pos
-                        frame_counter.inc()
-                        new_lastbins = process_fft(
-                            args,
-                            prom_vars,
-                            row.ts,
-                            fftbuffer,
-                            lastbins,
-                            running_df,
+                if max_sweep_start != last_sweep_start:
+                    if fftbuffer is None:
+                        frame_df = df
+                    else:
+                        frame_df = pd.concat(
+                            [fftbuffer, df[df["sweep_start"] == last_sweep_start]]
                         )
-                        if new_lastbins is not None:
-                            lastbins = new_lastbins
-                            if lastbins:
-                                lastbins_history = [lastbins] + lastbins_history
-                                lastbins_history = lastbins_history[: args.history]
-                            call_record_signals(args, lastbins_history, prom_vars)
-                        fftbuffer = []
-                        rotate_age = now - openlogts
-                        if rotate_age > args.rotatesecs:
-                            rotatelognow = True
-                        if runonce:
-                            return
-                    fftbuffer.append((row.ts, row.freq, row.pw))
+                        fftbuffer = df[df["sweep_start"] != last_sweep_start]
+                    last_sweep_start = max_sweep_start
+                    frame_counter.inc()
+                    logging.info(
+                        "frame with sweep_start %us ago",
+                        now - frame_df["sweep_start"].min(),
+                    )
+                    new_lastbins, last_df = process_fft(
+                        args,
+                        scan_config,
+                        prom_vars,
+                        frame_df,
+                        lastbins,
+                        running_df,
+                        last_dfs,
+                    )
+                    if args.nfftplots:
+                        last_dfs = last_dfs[-args.nfftplots :]
+                        last_dfs.append((last_df.freq, last_df.db))
+                    else:
+                        last_dfs = []
+                    if new_lastbins is not None:
+                        lastbins = new_lastbins
+                        if lastbins:
+                            lastbins_history = [lastbins] + lastbins_history
+                            lastbins_history = lastbins_history[: args.history]
+                        call_record_signals(args, lastbins_history, prom_vars)
+                    rotate_age = now - openlogts
+                    if rotate_age > args.rotatesecs:
+                        rotatelognow = True
+                else:
+                    if fftbuffer is None:
+                        fftbuffer = df
+                    else:
+                        fftbuffer = pd.concat([fftbuffer, df])
                 if rotatelognow:
                     break
         rotate_file_n(".".join((args.log, "zst")), args.nlog, require_initial=False)
@@ -432,28 +479,34 @@ def process_fft_lines(
         executor.submit(zstd_file, new_log)
 
 
-def fft_proxy(args, buff_file, buffer_time=FFT_BUFFER_TIME, shutdown_str=None):
+def fft_proxy(
+    args, buff_file, buffer_time=FFT_BUFFER_TIME, live_file=None, poll_timeout=1
+):
     zmq_addr = f"tcp://{args.logaddr}:{args.logport}"
     logging.info("connecting to %s", zmq_addr)
-    context = zmq.Context()
-    socket = context.socket(zmq.SUB)
+    zmq_context = zmq.Context()
+    socket = zmq_context.socket(zmq.SUB)
     socket.connect(zmq_addr)
     socket.setsockopt_string(zmq.SUBSCRIBE, "")
     packets_sent = 0
     last_packet_sent_time = time.time()
     tmp_buff_file = os.path.basename(buff_file)
     tmp_buff_file = buff_file.replace(tmp_buff_file, "." + tmp_buff_file)
-    shutdown = False
+    if os.path.exists(tmp_buff_file):
+        os.remove(tmp_buff_file)
     context = zstandard.ZstdCompressor()
+    shutdown = False
     while not shutdown:
         with open(tmp_buff_file, "wb") as zbf:
             with context.stream_writer(zbf) as bf:
                 while not shutdown:
-                    sock_txt = socket.recv()
+                    shutdown = live_file is not None and not live_file.exists()
+                    try:
+                        sock_txt = socket.recv(flags=zmq.NOBLOCK)
+                    except zmq.error.Again:
+                        time.sleep(poll_timeout)
+                        continue
                     bf.write(sock_txt)
-                    shutdown = (
-                        shutdown_str is not None and sock_txt.find(shutdown_str) != -1
-                    )
                     now = time.time()
                     if (
                         shutdown or now - last_packet_sent_time > buffer_time
@@ -466,10 +519,12 @@ def fft_proxy(args, buff_file, buffer_time=FFT_BUFFER_TIME, shutdown_str=None):
         os.rename(tmp_buff_file, buff_file)
 
 
-def find_signals(args, prom_vars):
-    with concurrent.futures.ProcessPoolExecutor(2) as executor:
-        proxy_result = executor.submit(fft_proxy, args, BUFF_FILE)
-        process_fft_lines(args, prom_vars, BUFF_FILE, executor, proxy_result)
+def find_signals(args, prom_vars, executor, live_file):
+    buff_file = os.path.join(args.buff_path, BUFF_FILE)
+    if os.path.exists(buff_file):
+        os.remove(buff_file)
+    proxy_result = executor.submit(fft_proxy, args, buff_file, live_file=live_file)
+    process_fft_lines(args, prom_vars, buff_file, executor, proxy_result, live_file)
 
 
 def argument_parser():
@@ -494,6 +549,9 @@ def argument_parser():
     parser.add_argument(
         "--nfftgraph", default=10, type=int, help="keep last N FFT graphs"
     )
+    parser.add_argument(
+        "--nfftplots", default=10, type=int, help="last N plots in FFT graphs"
+    )
     parser.add_argument(
         "--rotatesecs",
         default=3600,
@@ -506,12 +564,6 @@ def argument_parser():
     parser.add_argument(
         "--bin_mhz", default=20, type=int, help="monitoring bin width in MHz"
     )
-    parser.add_argument(
-        "--max_raw_power",
-        default=50,
-        type=float,
-        help="maximum raw power permitted from FFT",
-    )
     parser.add_argument(
         "--width",
         default=10,
@@ -559,18 +611,11 @@ def argument_parser():
         help="Prometheus client port",
     )
     parser.add_argument(
-        "--freq-end",
-        dest="freq_end",
-        type=float,
-        default=float(1e9),
-        help="Set freq_end [default=%(default)r]",
-    )
-    parser.add_argument(
-        "--freq-start",
-        dest="freq_start",
-        type=float,
-        default=float(100e6),
-        help="Set freq_start [default=%(default)r]",
+        "--port",
+        dest="port",
+        type=int,
+        default=80,
+        help="control webserver port",
     )
     parser.add_argument(
         "--logaddr",
@@ -600,7 +645,20 @@ def argument_parser():
         default=900,
         help="Number of seconds for running FFT average",
     )
-
+    parser.add_argument(
+        "--buff_path",
+        dest="buff_path",
+        type=str,
+        default="/dev/shm",  # nosec
+        help="Path for FFT buffer file",
+    )
+    parser.add_argument(
+        "--db_rolling_factor",
+        dest="db_rolling_factor",
+        type=float,
+        default=ROLLING_FACTOR,
+        help="Divisor for rolling dB average (or 0 to disable)",
+    )
     return parser
 
 
@@ -610,15 +668,6 @@ def main():
 
     logging.basicConfig(level=logging.DEBUG, format="%(asctime)s %(message)s")
     prom_vars = init_prom_vars()
-    start_http_server(args.promport)
-    x = threading.Thread(
-        target=find_signals,
-        args=(
-            args,
-            prom_vars,
-        ),
-    )
-    x.start()
     app = falcon.App()
     scanner_form = ScannerForm()
     result = Result()
@@ -626,4 +675,27 @@ def main():
     app.add_route("/", scanner_form)
     app.add_route("/result", result)
     app.add_route("/requests", active_requests)
-    bjoern.run(app, "0.0.0.0", 80)
+    start_http_server(args.promport)
+
+    with tempfile.TemporaryDirectory() as tmpdir:
+        live_file = pathlib.Path(os.path.join(tmpdir, "live_file"))
+        live_file.touch()
+
+        with concurrent.futures.ProcessPoolExecutor(2) as executor:
+            x = threading.Thread(
+                target=find_signals,
+                args=(
+                    args,
+                    prom_vars,
+                    executor,
+                    live_file,
+                ),
+            )
+            x.start()
+            try:
+                bjoern.run(app, "0.0.0.0", args.port)  # nosec
+            except KeyboardInterrupt:
+                logging.info("interrupt!")
+                live_file.unlink()
+                executor.shutdown()
+                x.join()
