@@ -62,7 +62,7 @@ def read_csv_chunks(args):
             df = df[
                 (df["frame"] < read_frames) & (df["freq"] >= minmhz)
             ]  # pylint: disable=unsubscriptable-object
-            df = calc_db(df)
+            df = calc_db(df, args.db_rolling_factor)
             df = df[df["db"] >= args.mindb]
             preprocess_frames(df)
             yield df
@@ -73,7 +73,7 @@ def read_csv_chunks(args):
         df = df[
             (df["frame"] < read_frames) & (df["freq"] >= minmhz)
         ]  # pylint: disable=unsubscriptable-object
-        df = calc_db(df)
+        df = calc_db(df, args.db_rolling_factor)
         df = df[df["db"] >= args.mindb]
         preprocess_frames(df)
         yield df
@@ -146,11 +146,11 @@ def freq_excluded(freq, freq_exclusions):
     return False
 
 
-def calc_db(df):
-    df["db"] = 20 * np.log10(df[df["db"] != 0]["db"])
+def calc_db(df, rolling_factor=ROLLING_FACTOR):
     df.replace([np.inf, -np.inf], np.nan, inplace=True)
     meandb = df["db"].mean()
-    df["db"] = df["db"].rolling(ROLLING_FACTOR).mean().fillna(meandb)
+    if rolling_factor:
+        df["db"] = df["db"].rolling(rolling_factor).mean().fillna(meandb)
     return df
 
 
@@ -160,7 +160,9 @@ def scipy_find_sig_windows(df, width, prominence, threshold):
     return [(df.iloc[peak].freq, df.iloc[peak].db) for peak in peaks]
 
 
-def graph_fft_peaks(graph_path, df, mean_running_df, sample_count_df, signals):
+def graph_fft_peaks(
+    graph_path, df, mean_running_df, sample_count_df, signals, last_dfs, scan_config
+):
     maxdb = df.db.max()
     df["peaks"] = df.db.min()
     for peak_freq, _ in signals:
@@ -189,6 +191,8 @@ def graph_fft_peaks(graph_path, df, mean_running_df, sample_count_df, signals):
         # sample_count_df["size"],
         # "c",
     )
+    for x, y in last_dfs:
+        plt.plot(x, y)
     plt.xlabel("freq (MHz)")
     plt.ylabel("power (dB)")
     plt.legend(("power", "peak status", "mean power"), loc="upper right")
@@ -197,9 +201,9 @@ def graph_fft_peaks(graph_path, df, mean_running_df, sample_count_df, signals):
     time_min = time.ctime(ts_min)
     time_max = time.ctime(ts_max)
     duration = ts_max - ts_min
-    plt.title(
-        f"gamutRF scanner FFT {time_min} to {time_max}, {duration}s\n{peak_signals}"
-    )
+    scan_config_txt = ", ".join([f"{x}: {y}" for x, y in scan_config.items()])
+    title = f"gamutRF scanner FFT {time_min} to {time_max}, {duration}s\n{scan_config_txt}\n{peak_signals}"
+    plt.title(title)
     real_path = os.path.realpath(graph_path)
     basename = os.path.basename(real_path)
     dirname = os.path.dirname(real_path)
